package game.mvc.model.enemy 
{
    import game.constants.Size;
    import game.mvc.model.circle.CircleModel;
    
    public class EnemyModel extends CircleModel
    {
        private static const MIN_SPEED:Number = 3;
        private static const MAX_SPEED:Number = 6;
        private static const DELTA_SPEED:Number = MAX_SPEED - MIN_SPEED;
        
        private static const LEFT:int = 0;
        private static const RIGHT:int = 1;
        private static const UP:int = 2;
        private static const DOWN:int = 3;
        
        private static const LEFT_LIMIT:int = -Size.ENEMY_RADIUS;
        private static const RIGHT_LIMIT:int = Size.GAME_WIDTH + Size.ENEMY_RADIUS;
        private static const UP_LIMIT:int = -Size.ENEMY_RADIUS;
        private static const DOWN_LIMIT:int = Size.GAME_HEIGHT + Size.ENEMY_RADIUS;
        
        private var _score:int;
        
        private var from:int;
        
        public function EnemyModel() 
        {
            init();
        }
        
        private function init():void 
        {
            _score = Math.random() * 10;
            from = Math.random() * 4;
            if (from == LEFT)
                initLeft();
            if (from == RIGHT)
                initRight();
            if (from == UP)
                initUp();
            if (from == DOWN)
                initDown();
        }
        
        private function initLeft():void
        {
            _x = -Size.ENEMY_RADIUS;
            _y = Size.ENEMY_RADIUS + Math.random() * (Size.GAME_HEIGHT - 2 * Size.ENEMY_RADIUS);
            speedX = getSpeed();
            speedY = 0;
        }
        
        private function initRight():void
        {
            _x = Size.GAME_WIDTH + Size.ENEMY_RADIUS;
            _y = Size.ENEMY_RADIUS + Math.random() * (Size.GAME_HEIGHT - 2 * Size.ENEMY_RADIUS);
            speedX = -getSpeed();
            speedY = 0;
        }
        
        private function initUp():void
        {
            _x = Size.ENEMY_RADIUS + Math.random() * (Size.GAME_WIDTH - 2 * Size.ENEMY_RADIUS);
            _y = -Size.ENEMY_RADIUS;
            speedX = 0;
            speedY = getSpeed();
        }
        
        private function initDown():void
        {
            _x = Size.ENEMY_RADIUS + Math.random() * (Size.GAME_WIDTH - 2 * Size.ENEMY_RADIUS);
            _y = Size.GAME_HEIGHT + Size.ENEMY_RADIUS;
            speedX = 0;
            speedY = -getSpeed();
        }
        
        private function getSpeed():Number
        {
            return MIN_SPEED + Math.random() * DELTA_SPEED;
        }
        
        override public function get isEnd():Boolean
        {
            if (from == LEFT && (_x > RIGHT_LIMIT)) return true;
            if (from == RIGHT && (_x < LEFT_LIMIT)) return true;
            if (from == UP && (_y > DOWN_LIMIT)) return true;
            if (from == DOWN && (_y < UP_LIMIT)) return true;
            return false;
        }
        
        public function get score():int 
        {
            return _score;
        }
    }
}