package game.mvc.model.tank 
{
    import game.constants.Size;
    
    public class TankModel 
    {
        private static const SPEED:Number = 6;
        private static const COS_SPEED:Number = SPEED * Math.cos(Math.PI / 4);
        
        private static const LEFT_LIMIT:int = Size.PLAYER_RADIUS;
        private static const RIGHT_LIMIT:int = Size.GAME_WIDTH - Size.PLAYER_RADIUS;
        private static const UP_LIMIT:int = Size.PLAYER_RADIUS;
        private static const DOWN_LIMIT:int = Size.GAME_HEIGHT - Size.PLAYER_RADIUS;
        
        private var _x:Number;
        private var _y:Number;
        private var _rotation:Number;
        
        public function TankModel() 
        {
            init();
        }
        
        private function init():void 
        {
            _x = Size.CENTER_X;
            _y = Size.CENTER_Y;
            _rotation = 0;
        }
        
        public function updateRotation(mouseX:Number, mouseY:Number):void
        {
            var deltaX:int = mouseX - _x;
            var deltaY:int = mouseY - _y;
            _rotation = Math.atan2(deltaY, deltaX);
        }
        
        public function moveLeft():void
        {
            _x -= SPEED;
            normalizeX();
        }
        
        public function moveRight():void
        {
            _x += SPEED;
            normalizeX();
        }
        
        public function moveUp():void
        {
            _y -= SPEED;
            normalizeY();
        }
        
        public function moveDown():void
        {
            _y += SPEED;
            normalizeY();
        }
        
        public function moveUpLeft():void
        {
            _x -= COS_SPEED;
            _y -= COS_SPEED;
            normalizeCoordinates();
        }
        
        public function moveUpRight():void
        {
            _x += COS_SPEED;
            _y -= COS_SPEED;
            normalizeCoordinates();
        }
        
        public function moveDownLeft():void
        {
            _x -= COS_SPEED;
            _y += COS_SPEED;
            normalizeCoordinates();
        }
        
        public function moveDownRight():void
        {
            _x += COS_SPEED;
            _y += COS_SPEED;
            normalizeCoordinates();
        }
        
        public function restart():void 
        {
            init();
        }
        
        private function normalizeCoordinates():void
        {
            normalizeX();
            normalizeY();
        }
        
        private function normalizeX():void
        {
            if (_x < LEFT_LIMIT) _x = LEFT_LIMIT;
            if (_x > RIGHT_LIMIT) _x = RIGHT_LIMIT;
        }
        
        private function normalizeY():void
        {
            if (_y < UP_LIMIT) _y = UP_LIMIT;
            if (_y > DOWN_LIMIT) _y = DOWN_LIMIT;
        }
        
        public function get x():Number 
        {
            return _x;
        }
        
        public function get y():Number 
        {
            return _y;
        }
        
        public function get rotation():Number 
        {
            return _rotation;
        }
    }
}