package game.mvc.views.game 
{
    import flash.display.Sprite;
    import game.mvc.model.BaseModel;
    import game.mvc.model.bullets.BulletModel;
    import game.mvc.model.enemy.EnemyModel;
    import game.mvc.views.game.background.InvisibleBackgroundView;
    import game.mvc.views.game.bullet.BulletView;
    import game.mvc.views.game.circle.CircleView;
    import game.mvc.views.game.enemy.EnemyView;
    import game.mvc.views.game.tank.TankView;
    
    public class GameView extends Sprite
    {
        private var invisibleBackground:InvisibleBackgroundView;
        private var enemies:Vector.<CircleView> = new Vector.<CircleView>();
        private var bullets:Vector.<CircleView> = new Vector.<CircleView>();
        private var tank:TankView;
        
        public function GameView(baseModel:BaseModel) 
        {
            invisibleBackground = new InvisibleBackgroundView();
            addChild(invisibleBackground);
            tank = new TankView(baseModel.tank);
            addChild(tank);
        }
        
        public function update():void
        {
            tank.update();
            updateCircles(enemies);
            updateCircles(bullets);
        }
        
        private function updateCircles(circles:Vector.<CircleView>):void
        {
            for (var i:int = 0; i < circles.length; i++)
            {
                var circleView:CircleView = circles[i];
                if (circleView.deleted)
                {
                    circles.splice(i, 1);
                    removeChild(circleView);
                    i--;
                }
                else
                    circleView.update();
            }
        }
        
        public function addEnemy(enemy:EnemyModel):void 
        {
            var enemyView:EnemyView = new EnemyView(enemy);
            enemies.push(enemyView);
            addChild(enemyView);
        }
        
        public function addBullet(bullet:BulletModel):void 
        {
            var bulletView:BulletView = new BulletView(bullet);
            bullets.push(bulletView);
            addChild(bulletView);
        }
        
        public function restart():void
        {
            deleteAllCircles(enemies);
            deleteAllCircles(bullets);
        }
        
        private function deleteAllCircles(circles:Vector.<CircleView>):void
        {
            for (var i:int = 0; i < circles.length; i++) 
                removeChild(circles[i]);
            circles.length = 0;
        }
    }
}